# Generated by Django 2.2 on 2022-02-12 16:14

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0002_auto_20220211_0730'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name='ConditionGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('type', models.IntegerField(default=1)),
                ('order', models.IntegerField(default=0)),
                ('condition_group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.ConditionGroup')),
                ('grid', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.Grid')),
            ],
        ),
        migrations.CreateModel(
            name='Condition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('value', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('criteria', models.IntegerField(default=1)),
                ('order', models.IntegerField(default=0)),
                ('condition_group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.ConditionGroup')),
            ],
        ),
    ]
