<div class="form-group">
    <div class="col-sm-4"><label>{{field.label}}</label></div>
    <div class="col-sm-8">
        {% if field.date_picker %}
        <div class='input-group date wide' id="{{field.full_name}}">
            <input name="{{field.full_name}}" type='text'
                   class="form-control"
                   value="{{field.value_as_str}}"
                   {% if field.placeholder %} placeholder="{{field.placeholder}}" {% endif %}
            />
            <span class="input-group-addon clickable">
                <span class="glyphicon glyphicon-calendar"></span>
            </span>
        </div>
        <script>
            $('#{{field.full_name}}').datetimepicker({
                locale: '{{field.date_picker_locale}}',
                format: '{{field.date_picker_format}}'
            });
        </script>
        {% else %}
        <input name="{{field.full_name}}" type="text"
               class="form-control"
               value="{{field.value_as_str}}"
               {% if field.placeholder %} placeholder="{{field.placeholder}}" {% endif %}
               {% if field.disabled %} disabled {% endif %}
        />
        {% endif %}
        {% for error in field.errors %}
        <div class="red">{{ error }}</div>
        {% endfor %}
    </div>
</div>