# Generated by Django 2.2 on 2023-12-18 14:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0012_auto_20230507_1342'),
    ]

    operations = [
    ]
